@@ -109,6 +109,8 @@ SEXP _RxODE_rinvchisq(SEXP, SEXP, SEXP);
 SEXP _RxODE_getRxFn(SEXP);
 SEXP _RxODE_setProgSupported(SEXP);
 SEXP _RxODE_getProgSupported();
+SEXP _RxODE_rxSetIni0(SEXP);
+SEXP _RxODE_rxSetSilentErr(SEXP silentSEXP);
 
 extern int rxIsCurrentC(SEXP obj);
 
@@ -207,6 +209,8 @@ void R_init_RxODE(DllInfo *info){
     {"_RxODE_rxSolve_", (DL_FUNC) &_RxODE_rxSolve_, 8},
     {"_RxODE_dropUnitsRxSolve", (DL_FUNC) &_RxODE_dropUnitsRxSolve, 1},
     {"_RxODE_atolRtolFactor_", (DL_FUNC) &_RxODE_atolRtolFactor_, 1},
+    {"_RxODE_rxSetIni0", (DL_FUNC) &_RxODE_rxSetIni0, 1},
+    {"_RxODE_rxSetSilentErr", (DL_FUNC) &_RxODE_rxSetSilentErr, 1},
     {NULL, NULL, 0}
   };
   // C callable to assign environments.
